#!/usr/bin/env python

import asyncio
import websockets
import threading
import functools
import json
import time
import tetristerminal as tetris

################################
### global connections. Must be complex data type. Simple integer will cause "UnboundLocalError"
CONNECTIONS = {"count": 0, "max": 4}

################################

async def register(websocket):
    "connection initiated. Set up game"

    if CONNECTIONS["count"]>=CONNECTIONS["max"]:
        print("register too busy. CONNECTIONS:",CONNECTIONS["count"])
        await websocket.send("!!!! register. Too busy for now. CONNECTIONS="+str(CONNECTIONS["count"])+" Try again later")
        return(None)
    
    CONNECTIONS["count"] += 1

    print("register CONNECTIONS:",CONNECTIONS["count"])
    await websocket.send("!!!! register. CONNECTIONS="+str(CONNECTIONS["count"]))

    ### Not this simple! When called in Tetris: RuntimeWarning: coroutine was never awaited. Inject below
    # async def mydrawfunction( todraw ):
    #     await( websocket.send(todraw) )

    ### construct new coroutine and inject into event_loop
    async def ws_send_server( thiswebsocket, data):
        await( thiswebsocket.send(data) )
    new_loop = asyncio.get_event_loop()
    def mydrawfunction( todraw ):
        asyncio.run_coroutine_threadsafe( ws_send_server(websocket, todraw), new_loop)

    ### start new game with mydrawfunction in new thread
    App = tetris.TetrisApp( mydrawfunction )
    App.init_game()
    gameThread= threading.Thread(target=App.run)
    gameThread.start()

    # game is now running. return reference to game so multiple can run simultaneously
    return(App)

async def unregister(websocket):
    "connection terminated"
    CONNECTIONS["count"] -= 1
    print("unregister CONNECTIONS:",CONNECTIONS["count"])

async def ws_handle_event( websocket, path ):
    print("ws_handle_event path",path)

    #### store the websocket being handled
    myapp = await register( websocket )

    #### if too busy then drop
    if myapp is None:
        return()

    #### main loop
    try:
        #### loop throught data in websocket async and handle
        async for data in websocket:
            if data == "left":
                myapp.STATE['value'] = "a"

            elif data == "rotate":
                myapp.STATE['value'] ="w"

            elif data == "right":
                myapp.STATE['value'] = "d"

            elif data == "drop":
                myapp.STATE['value'] = "s"

            elif data == "pause":
                myapp.STATE['value'] = "e"

            elif data == "quit":
                myapp.STATE['value'] = "q"

            elif "keypress" in data:
                myapp.STATE['value'] = data[-1] # key is at end of string

            else:
                print("ERR ws_handle_event",data)

            # update webpage for each message received
            #tosend = json.dumps(STATE)
            #print("sending",tosend)
            #await( websocket.send(tosend) )

    finally:
        await unregister(websocket)

################################
# serve any client on port 8000
myws = websockets.serve(ws_handle_event, '0.0.0.0', 8000)

def start_loop(loop, ws):
    asyncio.set_event_loop(loop)
    loop.run_until_complete(ws)
    loop.run_forever()
    
new_loop = asyncio.get_event_loop()
tt= threading.Thread(target=start_loop, args=(new_loop,myws))
tt.start()

#### Handle server side when thread handling webbrowser
print("In server thread")
print("Server thread nothing to do. Done! Could eliminate tt asyncio thread.")
