#!/usr/bin/env python
#-*- coding: utf-8 -*-

# Very simple tetris implementation
# 
# Control keys:
# Down - Drop stone faster
# Left/Right - Move stone
# Up - Rotate Stone clockwise
# Escape - Quit game
# P - Pause game
#
# Have fun!

# Copyright (c) 2010 "Kevin Chabowski"<kevin@kch42.de>
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from random import randrange as rand
import mygame, sys
import time

# The configuration
config = {
        'cell_size':    20,
        'cols':         8,
        'rows':         16,
        'delay':        750,
        'maxfps':       30
}

config = {
        'cell_size':    2,
        'cols':         8,
        'rows':         16,
        'delay':        750,
        'maxfps':       30
}

colors = [
(0,   0,   0  ),
(255, 0,   0  ),
(0,   150, 0  ),
(0,   0,   255),
(255, 120, 0  ),
(255, 255, 0  ),
(180, 0,   255),
(0,   220, 220)
]

colors = [
        " ",
        ".",
        "+",
        "*",
        "%",
        "@",
        "X",
        "O",
        "_"
]

# Define the shapes of the single parts
tetris_shapes = [
        [[1, 1, 1],
         [0, 1, 0]],
        
        [[0, 2, 2],
         [2, 2, 0]],
        
        [[3, 3, 0],
         [0, 3, 3]],
        
        [[4, 0, 0],
         [4, 4, 4]],
        
        [[0, 0, 5],
         [5, 5, 5]],
        
        [[6, 6, 6, 6]],
        
        [[7, 7],
         [7, 7]]
]

def rotate_clockwise(shape):
        return [ [ shape[y][x]
                        for y in range(len(shape)) ]
                for x in range(len(shape[0]) - 1, -1, -1) ]

def check_collision(board, shape, offset):
        off_x, off_y = offset
        for cy, row in enumerate(shape):
                for cx, cell in enumerate(row):
                        try:
                                if cell and board[ cy + off_y ][ cx + off_x ]:
                                        return True
                        except IndexError:
                                return True
        return False

def remove_row(board, row):
        del board[row]
        return [[0 for i in range(config['cols'])]] + board
        
def join_matrixes(mat1, mat2, mat2_off):
        off_x, off_y = mat2_off
        for cy, row in enumerate(mat2):
                for cx, val in enumerate(row):
                        mat1[cy+off_y-1 ][cx+off_x] += val
        return mat1

def new_board():
        board = [ [ 0 for x in range(config['cols']) ] for y in range(config['rows']-1) ]
        board += [[ 8 for x in range(config['cols'])]]
        return board

class TetrisApp(object):
        def __init__(self, drawfunction):
                self.width = config['cell_size']*config['cols']
                self.height = config['cell_size']*config['rows']
                self.screen = mygame.display_set_mode(self.width, self.height)
                self.screen.setDrawFunction( drawfunction )
                self.STATE={'value': None}
                
        def new_stone(self):
                self.stone = tetris_shapes[rand(len(tetris_shapes))]
                self.stone_x = int(config['cols'] / 2 - len(self.stone[0])/2)
                self.stone_y = 0
                
                if check_collision(self.board,
                                   self.stone,
                                   (self.stone_x, self.stone_y)):
                        self.gameover = True
                        self.screen.sendCmd("PLAY sound3") # gameover
                        
        def init_game(self):
                self.board = new_board()
                self.new_stone()
                self.gameover = False

        
        def draw_matrix(self, matrix, offset):
                off_x, off_y  = offset
                for y, row in enumerate(matrix):
                        for x, val in enumerate(row):
                                if val:
                                        mygame.draw_rect(
                                                self.screen,
                                                colors[val],
                                                (off_x+x) * config['cell_size'], (off_y+y) * config['cell_size'], 
                                                config['cell_size'], config['cell_size'])
        
        def move(self, delta_x):
                if not self.gameover and not self.paused:
                        new_x = self.stone_x + delta_x
                        if new_x < 0:
                                new_x = 0
                        if new_x > config['cols'] - len(self.stone[0]):
                                new_x = config['cols'] - len(self.stone[0])
                        if not check_collision(self.board,
                                               self.stone,
                                               (new_x, self.stone_y)):
                                self.stone_x = new_x
        def drop(self):
                if not self.gameover and not self.paused:
                        self.stone_y += 1
                        if check_collision(self.board,
                                           self.stone,
                                           (self.stone_x, self.stone_y)):
                                self.screen.sendCmd("PLAY sound2") # colliion on drop
                                self.board = join_matrixes(
                                  self.board,
                                  self.stone,
                                  (self.stone_x, self.stone_y))
                                self.new_stone()
                                while True:
                                        for i, row in enumerate(self.board[:-1]):
                                                if 0 not in row:
                                                        self.screen.sendCmd("PLAY sound1") # clear row

                                                        self.board = remove_row(self.board, i)
                                                        break
                                        else:
                                                break
        
        def rotate_stone(self):
                if not self.gameover and not self.paused:
                        new_stone = rotate_clockwise(self.stone)
                        if not check_collision(self.board,
                                               new_stone,
                                               (self.stone_x, self.stone_y)):
                                self.stone = new_stone
        
        def toggle_pause(self):
                self.paused = not self.paused
        
        def run(self):
                self.gameover = False
                self.paused = False

                timewaitNext = 1.0 # force drop every second
                timeFPS = 1.0/15.0 # 15 frames per second
                timebeginNext = time.time() # timer for forcing Next drop
                timebeginFrame = time.time() # timer for frame rate
                
                while not self.gameover:

                        #### allow user to control for timewaitNext, then drop
                        while (time.time()-timebeginNext) < timewaitNext:

                                if self.gameover:
                                        self.screen.settext("Done!")
                                        self.screen.draw()
                                else:
                                        if self.paused:
                                                self.screen.settext("Pause")
                                        else:
                                                self.screen.settext("None")
                                                self.screen.fill(" ")
                                                self.draw_matrix(self.board, (0,0))
                                                self.draw_matrix(self.stone,
                                                                 (self.stone_x,
                                                                  self.stone_y))

                                self.screen.draw()
                                
                                mykey = self.STATE['value'] 
                                self.STATE['value'] = None # clear for next
                                if mykey is not None:
                                        if mykey=="a": self.move(-1)
                                        if mykey=="d": self.move(1)
                                        if mykey=="w": self.rotate_stone()
                                        if mykey=="s": self.drop()
                                        if mykey=="q": self.gameover=True
                                        if mykey=="e": self.toggle_pause()
                                
                                #### wait for timeFPS to limit computation
                                tosleep = ( timeFPS - (time.time()-timebeginFrame) )
                                if tosleep>0.0: time.sleep(tosleep)
                                timebeginFrame = time.time()
                                
                        # drop one every timewait
                        self.drop()
                        if self.gameover:
                                self.screen.settext("Done!")
                                self.screen.draw()

                        timebeginNext=time.time()


if __name__ == '__main__':
        def drawfunction( todraw ):
                print(todraw)
        App = TetrisApp( drawfunction )
        App.init_game()
        App.run()
